@@ -3,11 +3,11 @@ package App::Netdisco::Core::Arpnip;
 use Dancer qw/:syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
-use App::Netdisco::Util::Node 'check_mac';
+use App::Netdisco::Util::PortMAC 'get_port_macs';
+use App::Netdisco::Util::SanityCheck 'check_mac';
 use App::Netdisco::Util::DNS ':all';
 use NetAddr::IP::Lite ':lower';
 use Time::HiRes 'gettimeofday';
-use NetAddr::MAC ();
 
 use base 'Exporter';
 our @EXPORT = ();
@@ -44,10 +44,12 @@ sub do_arpnip {
       return;
   }
 
+  my $port_macs = get_port_macs($device);
+
   # get v4 arp table
-  my $v4 = _get_arps($device, $snmp->at_paddr, $snmp->at_netaddr);
+  my @v4 = _get_arps($device, $port_macs, $snmp->at_paddr, $snmp->at_netaddr);
   # get v6 neighbor cache
-  my $v6 = _get_arps($device, $snmp->ipv6_n2p_mac, $snmp->ipv6_n2p_addr);
+  my @v6 = _get_arps($device, $port_macs, $snmp->ipv6_n2p_mac, $snmp->ipv6_n2p_addr);
 
   # get directly connected networks
   my @subnets = _gather_subnets($device, $snmp);
@@ -59,89 +61,71 @@ sub do_arpnip {
   my $now = 'to_timestamp('. (join '.', gettimeofday) .')';
 
   # update node_ip with ARP and Neighbor Cache entries
-  store_arp(\%$_, $now) for @$v4;
+  store_arp(@$_, $now) for @v4;
   debug sprintf ' [%s] arpnip - processed %s ARP Cache entries',
-    $device->ip, scalar @$v4;
+    $device->ip, scalar @v4;
 
-  store_arp(\%$_, $now) for @$v6;
+  store_arp(@$_, $now) for @v6;
   debug sprintf ' [%s] arpnip - processed %s IPv6 Neighbor Cache entries',
-    $device->ip, scalar @$v6;
+    $device->ip, scalar @v6;
 
   _store_subnet($_, $now) for @subnets;
   debug sprintf ' [%s] arpnip - processed %s Subnet entries',
     $device->ip, scalar @subnets;
-
-  $device->update({last_arpnip => \$now});
 }
 
 # get an arp table (v4 or v6)
 sub _get_arps {
-  my ($device, $paddr, $netaddr) = @_;
+  my ($device, $port_macs, $paddr, $netaddr) = @_;
   my @arps = ();
 
   while (my ($arp, $node) = each %$paddr) {
       my $ip = $netaddr->{$arp};
       next unless defined $ip;
-      next unless check_mac($device, $node);
-      push @arps, {
-        node => $node,
-        ip   => $ip,
-        dns  => undef,
-      };
+      next unless check_mac($device, $node, $port_macs);
+      push @arps, [$node, $ip, hostname_from_ip($ip)];
   }
 
-  debug sprintf ' resolving %d ARP entries with max %d outstanding requests',
-      scalar @arps, $ENV{'PERL_ANYEVENT_MAX_OUTSTANDING_DNS'};
-  my $resolved_ips = hostnames_resolve_async(\@arps);
-
-  return $resolved_ips;
+  return @arps;
 }
 
-=head2 store_arp( \%host, $now? )
+=head2 store_arp( $mac, $ip, $name, $now? )
 
 Stores a new entry to the C<node_ip> table with the given MAC, IP (v4 or v6)
-and DNS host name. Host details are provided in a Hash ref:
+and DNS host name.
 
- {
-    ip   => '192.0.2.1',
-    node => '00:11:22:33:44:55',
-    dns  => 'myhost.example.com',
- }
+Will mark old entries for this IP as no longer C<active>.
 
-The C<dns> entry is optional. The update will mark old entries for this IP as
-no longer C<active>.
-
-Optionally a literal string can be passed in the second argument for the
+Optionally a literal string can be passed in the fourth argument for the
 C<time_last> timestamp, otherwise the current timestamp (C<now()>) is used.
 
 =cut
 
 sub store_arp {
-  my ($hash_ref, $now) = @_;
+  my ($mac, $ip, $name, $now) = @_;
   $now ||= 'now()';
-  my $ip   = $hash_ref->{'ip'};
-  my $mac  = NetAddr::MAC->new($hash_ref->{'node'});
-  my $name = $hash_ref->{'dns'};
-
-  return if !defined $mac or $mac->errstr;
 
   schema('netdisco')->txn_do(sub {
     my $current = schema('netdisco')->resultset('NodeIp')
-      ->search(
-        { ip => $ip, -bool => 'active'},
-        { columns => [qw/mac ip/] })->update({active => \'false'});
+      ->search({ip => $ip, -bool => 'active'})
+      ->search(undef, {
+        columns => [qw/mac ip/],
+        order_by => [qw/mac ip/],
+        for => 'update'
+      });
+    $current->first; # lock rows
+    $current->update({active => \'false'});
 
     schema('netdisco')->resultset('NodeIp')
+      ->search({'me.mac' => $mac, 'me.ip' => $ip})
       ->update_or_create(
       {
-        mac => $mac->as_ieee,
-        ip => $ip,
         dns => $name,
         active => \'true',
         time_last => \$now,
       },
       {
-        key => 'primary',
+        order_by => [qw/mac ip/],
         for => 'update',
       });
   });
