@@ -3,15 +3,16 @@ package App::Netdisco::Web::Plugin::Search::Device;
 use Dancer ':syntax';
 use Dancer::Plugin::Ajax;
 use Dancer::Plugin::DBIC;
+use Dancer::Plugin::Auth::Extensible;
 
 use List::MoreUtils ();
 
 use App::Netdisco::Web::Plugin;
 
-register_search_tab({id => 'device', label => 'Device'});
+register_search_tab({ tag => 'device', label => 'Device' });
 
 # device with various properties or a default match-all
-ajax '/ajax/content/search/device' => sub {
+ajax '/ajax/content/search/device' => require_login sub {
     my $has_opt = List::MoreUtils::any {param($_)}
       qw/name location dns ip description model os_ver vendor/;
     my $set;
@@ -21,7 +22,7 @@ ajax '/ajax/content/search/device' => sub {
     }
     else {
         my $q = param('q');
-        return unless $q;
+        send_error('Missing query', 400) unless $q;
 
         $set = schema('netdisco')->resultset('Device')->search_fuzzy($q);
     }
